@@ -1,21 +1,21 @@
 """Library to handle connection with Tibber API."""
+from __future__ import annotations
+
 import asyncio
 import datetime as dt
 import logging
 import zoneinfo
-from typing import Optional
 
 import aiohttp
 import async_timeout
-from graphql_subscription_manager import SubscriptionManager
+from gql import Client
+from gql.transport.websockets import WebsocketsTransport
 
-from .const import __version__
+from .const import API_ENDPOINT, DEMO_TOKEN, __version__
 from .gql_queries import INFO, PUSH_NOTIFICATION
 from .tibber_home import TibberHome
 
 DEFAULT_TIMEOUT = 10
-DEMO_TOKEN = "5K4MVS-OjfWhK_4yrjOlFe1F6kJXPVf7eQYggo8ebAE"
-API_ENDPOINT = "https://api.tibber.com/v1-beta/gql"
 SUB_ENDPOINT = "wss://api.tibber.com/v1-beta/gql/subscriptions"
 
 _LOGGER = logging.getLogger(__name__)
@@ -24,75 +24,64 @@ _LOGGER = logging.getLogger(__name__)
 class Tibber:
     """Class to communicate with the Tibber api."""
 
-    # pylint: disable=too-many-instance-attributes
+    # pylint: disable=too-many-instance-attributes, too-many-arguments
 
     def __init__(
         self,
         access_token: str = DEMO_TOKEN,
         timeout: int = DEFAULT_TIMEOUT,
-        websession: Optional[aiohttp.ClientSession] = None,
-        time_zone: Optional[dt.tzinfo] = None,
+        websession: aiohttp.ClientSession | None = None,
+        time_zone: dt.tzinfo | None = None,
+        user_agent: str | None = None,
+        api_endpoint: str = API_ENDPOINT,
     ):
         """Initialize the Tibber connection.
 
         :param access_token: The access token to access the Tibber API with.
         :param websession: The websession to use when communicating with the Tibber API.
         :param time_zone: The time zone to display times in and to use.
+        :param user_agent: User agent identifier for the platform running this. Required if websession is None.
+        :param api_endpoint: Allow overriding API endpoint for easy testing
         """
+
         if websession is None:
-            self.websession = aiohttp.ClientSession(
-                headers={aiohttp.hdrs.USER_AGENT: f"pyTibber/{__version__}"}
-            )
+            websession = aiohttp.ClientSession()
         else:
-            self.websession = websession
+            user_agent = websession.headers.get(aiohttp.hdrs.USER_AGENT)
+        if user_agent is None:
+            raise Exception(
+                "Please provide value for HTTP user agent. Example: MyHomeAutomationServer/1.2.3"
+            )
+        self.user_agent = f"{user_agent} pyTibber/{__version__}"
+        self.websession = websession
+
         self._timeout: int = timeout
         self._access_token: str = access_token
         self.time_zone: dt.tzinfo = time_zone or zoneinfo.ZoneInfo("UTC")
         self._name: str = ""
-        self._user_id: Optional[str] = None
+        self._user_id: str | None = None
         self._active_home_ids: list[str] = []
         self._all_home_ids: list[str] = []
         self._homes: dict[str, TibberHome] = {}
-        self.sub_manager: Optional[SubscriptionManager] = None
-        try:
-            user_agent = self.websession._default_headers.get(
-                aiohttp.hdrs.USER_AGENT, ""
-            )  # will be fixed by aiohttp 4.0
-        except Exception:  # pylint: disable=broad-except
-            user_agent = ""
-        self.user_agent = f"{user_agent} pyTibber/{__version__}"
+        self.sub_manager: Client = self._get_sub_manager()
+        self.api_endpoint = api_endpoint
 
     async def close_connection(self) -> None:
         """Close the Tibber connection.
         This method simply closes the websession used by the object."""
         await self.websession.close()
 
-    async def rt_connect(self) -> None:
-        """Start the GraphQL subscription manager for real time data.
-        This method instantiates the graphql_subscription_manager.SubscriptionManager
-        class which authenticates the user with the provided access token, and then
-        starts the SubscriptionManager.
-        """
-        if self.sub_manager is not None:
-            return
-        self.sub_manager = SubscriptionManager(
-            {"token": self._access_token},
-            SUB_ENDPOINT,
-            self.user_agent,
-        )
-        self.sub_manager.start()
-
     async def rt_disconnect(self) -> None:
         """Stop subscription manager.
         This method simply calls the stop method of the SubscriptionManager if it is defined.
         """
-        if self.sub_manager is None:
+        if self.sub_manager.transport is None:
             return
-        await self.sub_manager.stop()
+        await self.sub_manager.transport.close()
 
     async def execute(
-        self, document: str, variable_values: Optional[dict] = None
-    ) -> Optional[dict]:
+        self, document: str, variable_values: dict | None = None
+    ) -> dict | None:
         """Execute a GraphQL query and return the data.
 
         :param document: The GraphQL query to request.
@@ -103,8 +92,8 @@ class Tibber:
         return res.get("data")
 
     async def _execute(
-        self, document: str, variable_values: dict = None, retry: int = 2
-    ) -> Optional[dict]:
+        self, document: str, variable_values: dict | None = None, retry: int = 2
+    ) -> dict | None:
         """Execute a GraphQL query and return the result as a dict loaded from the json response.
 
         :param document: The GraphQL query to request.
@@ -121,7 +110,7 @@ class Tibber:
         }
         try:
             async with async_timeout.timeout(self._timeout):
-                resp = await self.websession.post(API_ENDPOINT, **post_args)
+                resp = await self.websession.post(self.api_endpoint, **post_args)
             if resp.status != 200:
                 _LOGGER.error("Error connecting to Tibber, resp code: %s", resp.status)
                 return None
@@ -138,7 +127,7 @@ class Tibber:
             _LOGGER.error("Received non-compatible response %s", errors)
         return result
 
-    async def update_info(self, *_) -> None:
+    async def update_info(self) -> None:
         """Updates home info asynchronously."""
         if (res := await self._execute(INFO)) is None:
             return
@@ -166,13 +155,13 @@ class Tibber:
             if subs[0].get("status", "ended").lower() == "running":
                 self._active_home_ids += [home_id]
 
-    def get_home_ids(self, only_active=True) -> list[str]:
+    def get_home_ids(self, only_active: bool = True) -> list[str]:
         """Return list of home ids."""
         if only_active:
             return self._active_home_ids
         return self._all_home_ids
 
-    def get_homes(self, only_active: bool = True) -> list["TibberHome"]:
+    def get_homes(self, only_active: bool = True) -> list[TibberHome]:
         """Return list of Tibber homes."""
         return [
             home
@@ -180,7 +169,7 @@ class Tibber:
             if (home := self.get_home(home_id))
         ]
 
-    def get_home(self, home_id: str) -> Optional["TibberHome"]:
+    def get_home(self, home_id: str) -> TibberHome | None:
         """Return an instance of TibberHome for given home id."""
         if home_id not in self._all_home_ids:
             _LOGGER.error("Could not find any Tibber home with id: %s", home_id)
@@ -229,8 +218,27 @@ class Tibber:
                 tasks.append(home.fetch_production_data())
         await asyncio.gather(*tasks)
 
+    def _get_sub_manager(self):
+        return Client(
+            transport=WebsocketsTransport(
+                url=SUB_ENDPOINT,
+                init_payload={"token": self._access_token},
+                ping_interval=20,
+            ),
+        )
+
     @property
-    def user_id(self) -> Optional[str]:
+    def rt_subscription_running(self) -> bool:
+        """Is real time subscription running."""
+        return (
+            self.sub_manager.transport is not None
+            and isinstance(self.sub_manager.transport, WebsocketsTransport)
+            and self.sub_manager.transport.websocket is not None
+            and self.sub_manager.transport.websocket.open
+        )
+
+    @property
+    def user_id(self) -> str | None:
         """Return user id of user."""
         return self._user_id
 
